package net.wanji.web.service.scheme.impl;

import net.wanji.web.dto.CrossIdDTO;
import net.wanji.web.dto.DirListElement;
import net.wanji.web.dto.LaneListElement;
import net.wanji.web.dto.SaveLaneInfoDTO;
import net.wanji.web.mapper.scheme.CrossDirInfoMapper;
import net.wanji.web.mapper.scheme.CrossLaneLightsMapper;
import net.wanji.web.mapper.scheme.LaneInfoMapper;
import net.wanji.web.po.scheme.CrossDirInfoPO;
import net.wanji.web.po.scheme.LaneInfoPO;
import net.wanji.web.service.scheme.CrossConfigService;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2022/12/20 10:27
 */
@Service
public class CrossConfigServiceImpl implements CrossConfigService {
    private final CrossDirInfoMapper crossDirInfoMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final CrossLaneLightsMapper crossLaneLightsMapper;

    public CrossConfigServiceImpl(CrossDirInfoMapper crossDirInfoMapper, LaneInfoMapper laneInfoMapper, CrossLaneLightsMapper crossLaneLightsMapper) {
        this.crossDirInfoMapper = crossDirInfoMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.crossLaneLightsMapper = crossLaneLightsMapper;
    }

    @Override
    public void saveLaneInfo(SaveLaneInfoDTO saveLaneInfoDTO) {
        String crossId = saveLaneInfoDTO.getCrossId();
        List<DirListElement> dirList = saveLaneInfoDTO.getDirList();
        for (DirListElement dirListElement : dirList) {
            // 获取数据库中已有数据
            Integer dir = dirListElement.getDir();
            List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectByCrossIdAndDir(crossId, dir);
            // 按车道代码排序
            List<LaneListElement> laneListFromClient = dirListElement.getLaneList();
            laneListFromClient.sort((x,y) -> String.CASE_INSENSITIVE_ORDER.compare(x.getName(), y.getName()));
            List<LaneListElement> laneListFromDb = buildLaneListFromDb(laneInfoPOList);
            laneListFromDb.sort((x,y) -> String.CASE_INSENSITIVE_ORDER.compare(x.getName(), y.getName()));
            // 比较入参数据与已有数据
            if (!laneListFromClient.equals(laneListFromDb)) {
                // 删除车道表数据
                laneInfoMapper.deleteByCrossIdAndDir(crossId, dir);
                // 删除路口方向表数据
                crossDirInfoMapper.deleteInByDirType(crossId, dir);
                // 删除灯组-车道关系表数据
                List<String> laneIds = getLaneIds(laneInfoPOList);
                if (laneIds.size()!=0){
                    crossLaneLightsMapper.deleteByLaneIds(laneIds);
                }
                // 插入进口信息
                List<LaneInfoPO> laneInfoPOListForInsert = getLaneInfoPOListForInsert(
                        crossId, dirListElement, dir, laneListFromClient);
                laneInfoMapper.insertBatch(laneInfoPOListForInsert);
                // 插入路口方向
                CrossDirInfoPO crossDirInfoPO = getCrossDirInfoPO(crossId, dirListElement);
                crossDirInfoMapper.insertOne(crossDirInfoPO);
            }
            // 更新是否有行人道
            String id = getId(crossId, dirListElement);
            crossDirInfoMapper.updateIsPedestrian(dirListElement.getIsPersonCross(), id);
        }
    }

    @Override
    public SaveLaneInfoDTO listLaneInfo(CrossIdDTO crossIdDTO) {
        String crossId = crossIdDTO.getCrossId();
        SaveLaneInfoDTO saveLaneInfoDTO = new SaveLaneInfoDTO();
        saveLaneInfoDTO.setCrossId(crossId);
        saveLaneInfoDTO.setDirList(new ArrayList<>());
        List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectBycrossId(crossId);
        Map<Integer, List<LaneInfoPO>> collect = laneInfoPOList.stream()
                .collect(Collectors.groupingBy(LaneInfoPO::getDir));
        for (Map.Entry<Integer, List<LaneInfoPO>> entry : collect.entrySet()) {
            Integer key = entry.getKey();
            DirListElement dirListElement = new DirListElement();
            dirListElement.setDir(key);
            // 获取是否有行人道
            CrossDirInfoPO crossDirInfoPO = crossDirInfoMapper.selectByCrossIdAndDirType(crossId, key);
            Integer isPedestrian = crossDirInfoPO.getIsPedestrian();
            dirListElement.setIsPersonCross(isPedestrian);
            // 构造内层List
            List<LaneInfoPO> value = entry.getValue();
            List<LaneListElement> laneListElementList = new ArrayList<>();
            for (LaneInfoPO laneInfoPO : value) {
                LaneListElement laneListElement = new LaneListElement();
                laneListElement.setDirection(laneInfoPO.getTurn());
                laneListElement.setLaneType(laneInfoPO.getCategory());
                laneListElement.setName(laneInfoPO.getCode());
                laneListElementList.add(laneListElement);
            }
            dirListElement.setLaneList(laneListElementList);
            List<DirListElement> dirList = saveLaneInfoDTO.getDirList();
            dirList.add(dirListElement);
        }
        return saveLaneInfoDTO;
    }

    private static String getId(String crossId, DirListElement dirListElement) {
        CrossDirInfoPO crossDirInfoPO = new CrossDirInfoPO();
        Integer dir = dirListElement.getDir();
        String id = crossId + dir + 1 + 0;// todo 暂无主辅路序号，赋值0
        return id;
    }

    private static CrossDirInfoPO getCrossDirInfoPO(String crossId, DirListElement dirListElement) {
        CrossDirInfoPO crossDirInfoPO = new CrossDirInfoPO();
        Integer dir = dirListElement.getDir();
        crossDirInfoPO.setId(crossId + "_" + dir + "_" + 1 + "_" + 0); // todo 暂无主辅路序号，赋值0
        crossDirInfoPO.setDirType(dir);
        crossDirInfoPO.setInOutType(1);
        crossDirInfoPO.setCrossId(crossId);
        crossDirInfoPO.setLength(0D); // todo 暂无长度，赋值0
        crossDirInfoPO.setIsPedestrian(dirListElement.getIsPersonCross());
        return crossDirInfoPO;
    }

    private static List<LaneInfoPO> getLaneInfoPOListForInsert(String crossId, DirListElement dirListElement,
                                                               Integer dir, List<LaneListElement> laneListFromClient) {
        List<LaneInfoPO> laneInfoPOListForInsert = new ArrayList<>();
        for (LaneListElement laneListElement : laneListFromClient) {
            LaneInfoPO laneInfoPO = new LaneInfoPO();
            // 序号
            String name = laneListElement.getName(); // 01
            String substring = name.substring(name.length() - 1);// 1
            String s = 1 + substring;
            int sort = Integer.parseInt(s);
            laneInfoPO.setId(crossId + "_" + dirListElement.getDir() + "_" + sort);
            laneInfoPO.setCode(name);
            laneInfoPO.setSort(sort);
            laneInfoPO.setDir(dir);
            laneInfoPO.setTurn(laneListElement.getDirection());
            laneInfoPO.setCategory(laneListElement.getLaneType());
            laneInfoPO.setCrossId(crossId);
            laneInfoPOListForInsert.add(laneInfoPO);
        }
        return laneInfoPOListForInsert;
    }

    private List<String> getLaneIds(List<LaneInfoPO> laneInfoPOList) {
        List<String> laneIds = new ArrayList<>();
        for (LaneInfoPO laneInfoPO : laneInfoPOList) {
            String id = laneInfoPO.getId();
            laneIds.add(id);
        }
        return laneIds;
    }

    private List<LaneListElement> buildLaneListFromDb(List<LaneInfoPO> laneInfoPOList) {
        List<LaneListElement> laneListElements = new ArrayList<>();
        for (LaneInfoPO laneInfoPO : laneInfoPOList) {
            LaneListElement laneListElement = new LaneListElement();
            laneListElement.setDirection(laneInfoPO.getTurn());
            laneListElement.setName(laneInfoPO.getCode());
            laneListElement.setLaneType(laneInfoPO.getCategory());
            laneListElements.add(laneListElement);
        }
        return laneListElements;
    }
}
